import fugashi
import srt
import re
from collections import Counter
from tqdm import tqdm
from pathlib import Path
from zipfile import ZipFile
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor

class DictMakerMT:
    """docstring for DictMaker"""

    def __init__(self, threads=4):
        self.tagger = fugashi.Tagger()
        if self.tagger.dictionary_info[0]["size"] < 872000:
            print("Please execute 'python -m unidic download'")
            raise ImportError
        self.wcounter = Counter()
        self.refcounter = Counter()
        self.hiragana = set(
            "あえいおうかけきこくさしすせそたちつてとなにぬねのはひふへほ"
            "まみむめもやゆよらりるれろわをんがぎぐげござじずぜぞだぢづ"
            "でどばびぶべぼぱぴぷぺぽゃょゅっぁぃぉぅぇゎゝゐゑゔ"
        )

        self.katakana = set(
            "アエイオウカケキコクサシスセソタチツテトナニヌネノハヒフヘホ"
            "マミムメモヤユヨラリルレロワヲウンガギグゲゴザジズゼゾダヂヅ"
            "デドバビブベボパピプペポょャュィョェァォッーゥヮヴヵヶﾘｫｶｯｮｼｵﾌｷﾏﾉﾀ"
        )
        self.sentence_marker = set("。、!！？」「』『（）〝〟)(\n")
        self.all_kanji_set = set(
            chr(uni) for uni in range(ord("一"), ord("龯") + 1)
        ) | set("〆々")
        self.allowed = (
            self.all_kanji_set | self.sentence_marker | self.katakana | self.hiragana
        )
        self.threads = threads

    def _clean_html(self, text):
        text = re.sub(r'<font size="1">(.*?)<\/font>', "", text)
        text = re.sub(r"<rt>(.*?)<\/rt>", "", text)
        text = "".join(filter(self.allowed.__contains__, text))
        text = re.sub(r"\n+", "\n", text)
        return text

    def _clean_txt(self, text):
        text = "".join(filter(self.allowed.__contains__, text))
        text = re.sub(r"\n+", "\n", text)
        return text

    def _clean_srt(self, text):
        subs = list(srt.parse(text))

        def remove_names(text):
            return re.sub(r"\（(.*?)\）", "", text)

        text = "".join(f"{remove_names(s.content)}\n" for s in subs)
        return self._clean_txt(text)

    def normalize_refcounter(self, value):
        """Use this if you haven't normalized the Counter before due to feeding several file lists or single texts"""
        for key in self.refcounter.keys():
            self.refcounter[key] /= value

    def feed_text(self, text, refcounter_add=False):
        total_tokens = list()
        for sen in text.split("\h"):
            sen_tokens = [
                w.feature.lemma.split("-")[0] if w.feature.lemma else w.surface
                for w in self.tagger(sen)
            ]
            sen_tokens = [
                w
                for w in sen_tokens
                if not (
                    w in self.hiragana
                    or w in self.katakana
                    or w in self.sentence_marker
                )
            ]
            total_tokens.extend(sen_tokens)
        if refcounter_add:
            self.refcounter.update(set(total_tokens))
        self.wcounter.update(total_tokens)

    def feed_files(self, filelist, reset_refcounter=True, normalize_refcounter=True):
        if reset_refcounter:
            self.refcounter = Counter()
        for entry in tqdm(filelist):
            file = Path(entry)
            if file.suffix == ".html":
                with open(file, "r", encoding="utf-8") as f:
                    text = self._clean_html(f.read())
            elif file.suffix == ".txt":
                with open(file, "r", encoding="utf-8") as f:
                    text = self._clean_txt(f.read())
            elif file.suffix == ".srt":
                with open(file, "r", encoding="utf-8") as f:
                    text = self._clean_srt(f.read())
            else:
                print(f"unable to process {entry} due to it being {file.suffix}")
                raise FileNotFoundError
            self.feed_text(text, refcounter_add=True)
        # normalize refcounter
        if normalize_refcounter:
            total = len(filelist)
            for key in self.refcounter.keys():
                self.refcounter[key] /= total
    def process_file(self, entry):
        file = Path(entry)
        if file.suffix == ".html":
            with open(file, "r", encoding="utf-8") as f:
                text = self._clean_html(f.read())
        elif file.suffix == ".txt":
            with open(file, "r", encoding="utf-8") as f:
                text = self._clean_txt(f.read())
        elif file.suffix == ".srt":
            with open(file, "r", encoding="utf-8") as f:
                text = self._clean_srt(f.read())
        else:
            print(f"unable to process {entry} due to it being {file.suffix}")
            raise FileNotFoundError
        self.feed_text(text, refcounter_add=True)

    def feed_filesMT(self, filelist, reset_refcounter=True, normalize_refcounter=True):
        if reset_refcounter:
            self.refcounter = Counter()
        with ProcessPoolExecutor(max_workers=self.threads) as executor:
            for arg, res in tqdm(zip(filelist, executor.map(self.process_file, filelist))):
                pass
        if normalize_refcounter:
            total = len(filelist)
            for key in self.refcounter.keys():
                self.refcounter[key] /= total

    def save(self, filepath, dictname):
        fpath = Path(filepath)
        if fpath.suffix != ".zip":
            fpath = fpath.parent / (fpath.name + ".zip")
        yomi_title = '{"title":"' + dictname + '_W","format":3,"revision":"frequency1"}'
        freqstr = ""
        idx = 1
        for tok in self.wcounter.most_common():
            freqstr += f'["{tok[0]}","freq"," {idx} F: {tok[1]} %: {self.refcounter.get(tok[0],0)*100:.2f}"],'
            idx += 1
        freqstr = "[" + freqstr[:-1] + "]"
        with ZipFile(fpath, "w") as zipf:
            zipf.writestr("index.json", yomi_title)
            zipf.writestr("term_meta_bank_1.json", freqstr)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yomidict']

package_data = \
{'': ['*']}

install_requires = \
['EbookLib>=0.17.1,<0.18.0',
 'fugashi>=1.1.0,<2.0.0',
 'srt>=3.4.1,<4.0.0',
 'unidic>=1.0.3,<2.0.0']

setup_kwargs = {
    'name': 'yomidict',
    'version': '0.1.5',
    'description': 'Create frequency dictionaries for yomichan out of media',
    'long_description': '# yomidict\nCreate frequency dictionaries for yomichan out of media.\\\nCurrently supported formats are: epub, html, srt, txt\n\n```python\npip install yomidict\n```\n\n\nMWE:\n```python\nimport yomidict\ndm = yomidict.DictMaker()\nfilelist = ["test.html"]*5 + ["test.epub"]*2 + ["test.srt"]*2\ndm.feed_files(filelist)\ndm.save("zipfile.zip", "name_in_yomichan", use_suffix=True)\n```\n\n# Docs:\n\n## DictMaker Object\n### wcounter\nIs a Counter which saves the number of occurences for the tokens that were found during feeding.\n\n### refcounter\nKeeps track of in how many files a certain token was found. E.g. a value of 0.5 (if normalized) would mean that the token occurs in 50% of all files that were fed.\n\n## DictMaker.feedfiles()\n```python\ndef feed_files(\n        self,\n        filelist,\n        skip_errors=False,\n        reset_refcounter=True,\n        normalize_refcounter=True,\n    )\n```\nskip_erros: does exactly as the name suggests, it skips errors. During processing of a bunch of files all sorts of errors could occur which would abort the feeding. This might be undesirable and so they can be skipped. The errored files will also be taken in consideration when calculating the DictMaker.refcounter.\n\nreset_refcounter: resets the refcounter before feeding files.\n\nnormalize_refcounter: count/total_number_of_files. Therefore, if a token comes up in 8 out of 10 books the value of the counter would be 0.8 instead of 8. This makes it easier to read even without knowing the total number of files that were fed into DictMaker.\n\n## DictMaker.save()\n```python\ndef save(\n        self,\n        filepath,\n        dictname,\n        only_rank_and_freq=False,\n        use_suffix=False,\n        use_suffix_rank=False,\n        use_suffix_freq=False,\n    )\n```\nonly_rank_and_freq: by default it the word rank, the word frequency and the refcounter_value get saved. This deactivates the refcounter_value.\n\nuse_suffix: activates use_suffix_rank and use_suffix_freq.\n\nuse_suffix_rank: if the number is above 1000 the number gets replaced by "num/1000 K" e.g. 2530 becomes 2K and 2434455 becomes 2M.\n\nuse_suffix_freq: same as use_suffix_freq but for the frequency\n\n## DictMaker.feed_text()\n```python\ndef feed_text(self, text, refcounter_add=False)\n```\ncan be used to feed a string into DictMaker.\n\nrefcounter_add: If true it adds 1 occurrence in refcounter to all the tokens that were found in the fed text.\n\n### How to feed a large text file\n\nDo you want to use refcounter? If yes, do you know the number of works inside the large text file? No? Don\'t use refcounter.\n\nIf you do know the number of works inside the large text file, do you know where one work ends and the other begins? Nice, just read it as chunks and let it add to the refcounter and normalize it in the end. If not, don\'t use refcounter.\n\nTo feed a large text file you can just read the text file line by line or sentence by sentence and utilizie the `DictMaker._clean_txt()` function.\n\n```python\ndm = yomidict.DictMaker()\nfor line in large_txt_file:\n    dm.feed_text(dm._clean_txt(line))\n```\n\nIf you know the boundaries of each work and can it eat in chunks you could something like this:\n\n```python\ndm = yomidict.DictMaker()\nfor work in large_txt_file:\n    dm.feed_text(dm._clean_txt(work), refcounter_add=True)\ndm.normalize_refcounter(works_in_large_txt_file)\n```\n',
    'author': 'exc4l',
    'author_email': 'cps0537@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/exc4l/yomidict',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import fugashi
import srt
import re
import ebooklib
from collections import Counter
from tqdm import tqdm
from pathlib import Path
from zipfile import ZipFile
from ebooklib import epub


class DictMaker:
    """docstring for DictMaker"""

    def __init__(self):
        self.tagger = fugashi.Tagger()
        if self.tagger.dictionary_info[0]["size"] < 872000:
            print("Please execute 'python -m unidic download'")
            raise ImportError("Please execute 'python -m unidic download'")
        self.wcounter = Counter()
        self.refcounter = Counter()
        self.hiragana = set(
            "あえいおうかけきこくさしすせそたちつてとなにぬねのはひふへほ"
            "まみむめもやゆよらりるれろわをんがぎぐげござじずぜぞだぢづ"
            "でどばびぶべぼぱぴぷぺぽゃょゅっぁぃぉぅぇゎゝゐゑゔ"
        )

        self.katakana = set(
            "アエイオウカケキコクサシスセソタチツテトナニヌネノハヒフヘホ"
            "マミムメモヤユヨラリルレロワヲウンガギグゲゴザジズゼゾダヂヅ"
            "デドバビブベボパピプペポょャュィョェァォッーゥヮヴヵヶﾘｫｶｯｮｼｵﾌｷﾏﾉﾀ"
        )
        self.sentence_marker = set("。、!！？」「』『（）〝〟)(\n")
        self.all_kanji_set = set(
            chr(uni) for uni in range(ord("一"), ord("龯") + 1)
        ) | set("〆々")
        self.allowed = (
            self.all_kanji_set | self.sentence_marker | self.katakana | self.hiragana
        )

    def _clean_html(self, text):
        text = re.sub(r'<font size="1">(.*?)<\/font>', "", text)
        text = re.sub(r"<rt>(.*?)<\/rt>", "", text)
        text = "".join(filter(self.allowed.__contains__, text))
        text = re.sub(r"\n+", "\n", text)
        return text

    def _clean_txt(self, text):
        text = "".join(filter(self.allowed.__contains__, text))
        text = re.sub(r"\n+", "\n", text)
        return text

    def _clean_srt(self, text):
        subs = list(srt.parse(text))

        def remove_names(text):
            return re.sub(r"\（(.*?)\）", "", text)

        text = "".join(f"{remove_names(s.content)}\n" for s in subs)
        return self._clean_txt(text)

    def _clean_epub(self, file):
        book = epub.read_epub(file)
        text = str()
        for doc in book.get_items_of_type(ebooklib.ITEM_DOCUMENT):
            text += str(doc.content.decode("utf-8-sig")) + "\n"
        return self._clean_html(text)

    def normalize_refcounter(self, value):
        """Use this if you haven't normalized the Counter before due to feeding several file lists or single texts"""
        for key in self.refcounter.keys():
            self.refcounter[key] /= value

    def feed_text(self, text, refcounter_add=False):
        total_tokens = list()
        for sen in text.split("\n"):
            sen_tokens = [
                w.feature.lemma.split("-")[0] if w.feature.lemma else w.surface
                for w in self.tagger(sen)
            ]
            sen_tokens = [
                w
                for w in sen_tokens
                if not (
                    w in self.hiragana
                    or w in self.katakana
                    or w in self.sentence_marker
                )
            ]
            total_tokens.extend(sen_tokens)
        if refcounter_add:
            self.refcounter.update(set(total_tokens))
        self.wcounter.update(total_tokens)

    def feed_files(
        self,
        filelist,
        skip_errors=False,
        reset_refcounter=True,
        normalize_refcounter=True,
    ):
        if reset_refcounter:
            self.refcounter = Counter()
        failed_files = 0
        for entry in tqdm(filelist):
            file = Path(entry)
            try:
                if file.suffix == ".html":
                    with open(file, "r", encoding="utf-8") as f:
                        text = self._clean_html(f.read())
                elif file.suffix == ".txt":
                    with open(file, "r", encoding="utf-8") as f:
                        text = self._clean_txt(f.read())
                elif file.suffix == ".srt":
                    with open(file, "r", encoding="utf-8") as f:
                        text = self._clean_srt(f.read())
                elif file.suffix == ".epub":
                    text = self._clean_epub(file)
                else:
                    print(
                        f"\nUnable to process {entry} due to being a {file.suffix} file"
                    )
                    if skip_errors:
                        failed_files += 1
                        continue
                    raise TypeError(f"{file.suffix} parser not implemented")
                self.feed_text(text, refcounter_add=True)
            except Exception as e:
                print(f"\nFailed to process {entry}: {e}")
                failed_files += 1
                if not skip_errors:
                    raise e
        # normalize refcounter
        if normalize_refcounter:
            total = len(filelist) - failed_files
            for key in self.refcounter.keys():
                self.refcounter[key] /= total

    def save(
        self,
        filepath,
        dictname,
        only_rank_and_freq=False,
        use_suffix=False,
        use_suffix_rank=False,
        use_suffix_freq=False,
    ):
        def suffix_numbers(number):
            if number > 1e6 - 1:
                return f"{int(number/1e6)}M"
            if number > 1e3 - 1:
                return f"{int(number/1e3)}K"
            return f"{number}"

        def get_num(num, use_suffix):
            if use_suffix:
                return suffix_numbers(num)
            return num

        if use_suffix:
            use_suffix_rank = True
            use_suffix_freq = True
        fpath = Path(filepath)
        if fpath.suffix != ".zip":
            fpath = fpath.parent / (fpath.name + ".zip")
        yomi_title = '{"title":"' + dictname + '","format":3,"revision":"frequency1"}'
        freqlist = list()
        idx = 1
        for tok in self.wcounter.most_common():
            if only_rank_and_freq:
                freqlist.append(
                    f'["{tok[0]}","freq","W: {get_num(idx,use_suffix_rank)} F: {get_num(tok[1], use_suffix_freq)}"],'
                )
            else:
                freqlist.append(
                    f'["{tok[0]}","freq","W: {get_num(idx,use_suffix_rank)} F: {get_num(tok[1], use_suffix_freq)} %: {self.refcounter.get(tok[0],0)*100:.2f}"],'
                )
            idx += 1
        freqstr = "".join(freqlist)
        freqstr = "[" + freqstr[:-1] + "]"
        with ZipFile(fpath, "w") as zipf:
            zipf.writestr("index.json", yomi_title)
            zipf.writestr("term_meta_bank_1.json", freqstr)
